@@ -8,7 +8,9 @@ class TestUtil(unittest.TestCase):
         self.pkcs11.load()
 
         # get SoftHSM major version
-        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]
+        info = self.pkcs11.getInfo()
+        self.SoftHSMversion = info.libraryVersion[0]
+        self.manufacturer = info.manufacturerID
 
         self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
         self.session = self.pkcs11.openSession(
@@ -151,6 +153,30 @@ class TestUtil(unittest.TestCase):
 
         self.assertEqual(text, plainText)
 
+    def test_RSA_OAEPwithAAD(self):
+        # AAD is "Additional Authentication Data"
+        # (pSourceData of CK_RSA_PKCS_OAEP_PARAMS struct)
+        if self.SoftHSMversion < 2:
+            self.skipTest("RSA OAEP only supported by SoftHSM >= 2")
+
+        if self.manufacturer.startswith("SoftHSM"):
+            # SoftHSM indicates in syslog:
+            #  "SoftHSM.cpp(12412): pSourceData must be NULL"
+            # and returns CKR_ARGUMENTS_BAD
+            self.skipTest("'AAD' not (yet) supported.")
+
+        plainText = "A test string"
+
+        # RSA OAEP
+        aad = "sample aad".encode("utf-8")
+        mech = PyKCS11.RSAOAEPMechanism(PyKCS11.CKM_SHA_1, PyKCS11.CKG_MGF1_SHA1, aad)
+        cipherText = self.session.encrypt(self.pubKey, plainText, mech)
+        decrypted = self.session.decrypt(self.privKey, cipherText, mech)
+
+        text = bytes(decrypted).decode("utf-8")
+
+        self.assertEqual(text, plainText)
+
     def test_RSA_PSS_SHA1(self):
         if self.SoftHSMversion < 2:
             self.skipTest("RSA PSS only supported by SoftHSM >= 2")
